@@ -1,22 +1,21 @@
 package CGI::Builder::HTMLtmpl ;
-$VERSION = 1.1 ;
+$VERSION = '1.2' ;
 
 # This file uses the "Perlish" coding style
 # please read http://perl.4pro.net/perlish_coding_style.html
 
 ; use strict
 ; use HTML::Template
+; use File::Spec
 ; $Carp::Internal{'HTML::Template'}++
 ; $Carp::Internal{+__PACKAGE__}++
 
 ; my $print_code
 ; BEGIN
-   { $print_code = sub
-                    { shift()->CGI::Builder::HTMLtmpl::ht_print(@_)
-                    }
+   { $print_code = sub{ shift()->CGI::Builder::HTMLtmpl::_::ht_print(@_) }
    }
 
-   ; use Object::props
+; use Object::props
       ( { name       => 'ht'
         , default    => sub{ shift()->ht_new(@_) }
         }
@@ -28,39 +27,67 @@ $VERSION = 1.1 ;
         }
       )
       
-; use Object::groups
-      ( { name       => 'ht_new_args'
-        , default    =>  sub
-                          { return
-                            { filename => $_[0]->page_name
-                                        . $_[0]->page_suffix
-                            , path     => [ $_[0]->page_path ]
-                            }
-                          }
-        }
-      )
+; use Object::groups qw| ht_new_args ht_param |
 
 ; sub ht_new
-   { HTML::Template->new( %{$_[0]->ht_new_args} )
+   { my $s = shift
+   ; HTML::Template->new( filename => $s->page_name . $s->page_suffix
+                        , path     => [ $s->page_path ]
+                        , %{$s->ht_new_args}
+                        )
    }
 
 ; sub page_content_check
    { my $s  = shift
-   ; unless ( $s->page_content eq $print_code )  # not managed by ht
-      { $s->CGI::Builder::page_content_check
-      }
-     else
-      { my $template = $s->page_name
-                     . $s->page_suffix
-      # check
-      ; $s->ht->_find_file( $template )
-      }
+   ; $s->page_content eq $print_code   # managed by ht
+     ? $s->CGI::Builder::HTMLtmpl::_::ht_exists
+     : $s->CGI::Builder::page_content_check
    }
 
-; sub CGI::Builder::HTMLtmpl::ht_print
-   { print shift()->ht->output()
+; sub CGI::Builder::HTMLtmpl::_::ht_print
+   { my $s = shift
+   ; $s->ht->param( %{$s->ht_param} )
+   ; print $s->ht->output()
    }
    
+# adapted from HTML::Template::_find_file
+# it implements the same logic
+
+; sub CGI::Builder::HTMLtmpl::_::ht_exists
+   { my $s = shift
+   ; my $filename =  $$s{ht_new_args}{filename}
+                  || $s->page_name.$s->page_suffix
+   ; my $path     =  $$s{ht_new_args}{path}
+                  || [ $s->page_path ]
+   # first check for a full path
+   ; if ( File::Spec->file_name_is_absolute( $filename ) )
+      { return 1 if -e $filename
+      }
+   # try pre-prending HTML_Template_Root
+   ; if ( exists $ENV{HTML_TEMPLATE_ROOT} )
+      { return 1 if -e File::Spec->catfile( $ENV{HTML_TEMPLATE_ROOT}
+                                          , $filename
+                                          )
+      }
+   # try "path" option list..
+   ; foreach my $p ( @$path )
+      { return 1 if -e File::Spec->catfile( $p
+                                          , $filename
+                                          )
+      }
+   # try even a relative path from the current directory...
+   ; return 1 if -e $filename
+   # try "path" option list with HTML_TEMPLATE_ROOT prepended...
+   ; if ( exists $ENV{HTML_TEMPLATE_ROOT} )
+      { foreach my $p ( @$path )
+         { return 1 if -e File::Spec->catfile( $ENV{HTML_TEMPLATE_ROOT}
+                                             , $p
+                                             , $filename
+                                             )
+         }
+      }
+   ; return undef
+   }
       
 ; 1
    
@@ -70,7 +97,7 @@ __END__
 
 CGI::Builder::HTMLtmpl - CGI::Builder and HTML::Template integration
 
-=head1 VERSION 1.1
+=head1 VERSION 1.2
 
 To have the complete list of all the extensions of the CBF, see L<CGI::Builder/"Extensions List">
 
@@ -121,7 +148,7 @@ B<Note>: With this extension you don't need to explicitly set the C<page_content
     # usually you just need to pass the param to the object
     sub PH_myPage {
         my $s = shift;
-        $s->ht->param( something => 'something' );
+        $s->ht_param( something => 'something' );
     }
     
     # but if you want to completely bypass the template system
@@ -131,7 +158,7 @@ B<Note>: With this extension you don't need to explicitly set the C<page_content
         $s->page_content = 'some content';
     }
 
-B<Note>: This extension is not as magic and memory saving as the L<CGI::Builder::Magic|CGI::Builder::Magic> template extension, because HTML::Template requires a specific input data structure (i.e. does not allow call back subs unless you use the HTML::Template::Expr), and does not allow to print the output during the process. On the other hand it should be some milliseconds faster than CGI::Builder::Magic in producing the output.
+B<Note>: This extension is not as magic and memory saving as the L<CGI::Builder::Magic|CGI::Builder::Magic> template extension, because HTML::Template requires a specific input data structure (i.e. does not allow call back subs unless you use the HTML::Template::Expr), and does not allow to print the output during the process. On the other hand it should be a few milliseconds faster than CGI::Builder::Magic in producing the output.
 
 =head1 EXAMPLES
 
@@ -146,8 +173,8 @@ This is a complete CBB that uses all the default to load the './tm/index.tmpl'te
       
     sub PH_index {
         my $s = shift;
-        $s->ht->param( myVar      => 'my Variable',
-                       myOtherVar => 'other Variable');
+        $s->ht_param( myVar      => 'my Variable',
+                      myOtherVar => 'other Variable');
     }
     
     1;
@@ -166,7 +193,7 @@ This is a more complex complete CBB that will automatically send the C<page_cont
     sub OH_init {
         my $s = shift;
         $s->page_suffix = '.html';               # override defaults
-        $s->ht_new_args( path => '/my/path',     # override defaults
+        $s->ht_new_args( path => ['/my/path'],   # override defaults
                          die_on_bad_params => 0,
                          cache => 1 );
     }
@@ -176,8 +203,8 @@ This is a more complex complete CBB that will automatically send the C<page_cont
     # and will fill it with the following variables and send the output()
     sub PH_index {
         my $s = shift;
-        $s->ht->param( myVar => 'my Variable',
-                       myOtherVar => 'other Variable');
+        $s->ht_param( myVar      => 'my Variable',
+                      myOtherVar => 'other Variable');
     }
     
     # this will override the default template for this handler
@@ -186,7 +213,7 @@ This is a more complex complete CBB that will automatically send the C<page_cont
     sub PH_specialPage {
         my $s = shift;
         $s->ht_new_args( filename => 'special.tmp')     # override defaults
-        $s->ht->param( mySomething => 'something' );
+        $s->ht_param( mySomething => 'something' );
     }
     
     1;
@@ -195,12 +222,6 @@ This is a more complex complete CBB that will automatically send the C<page_cont
 
 This module adds some template properties (all those prefixed with 'ht_') to the standard CBF properties. The default of these properties are usually smart enough to do the right job for you, but you can fine-tune the behaviour of your CBB by setting them to the value you need.
 
-=head2 ht
-
-This property returns the internal C<HTML::Template> object that is automatically created just before you use it. Use it to address all the HTML::Template methods.
-
-B<Note>: You can change the default arguments that are internally used to create the object by using the C<ht_new_args> group accessor, or you can completely override the creation of the internal object by overriding the C<ht_new()> method.
-
 =head2 ht_new_args( arguments )
 
 This property group accessor handles the HTML::Template constructor arguments that are used in the creation of the internal HTML::Template object. Use it to change or add the argument you need to the creation of the new object.
@@ -224,15 +245,15 @@ This option is set to the C<page_name> value plus the C<page_suffix> value.
 
 This option is set to the C<page_path> value.
 
-   path => $s->page_path
+   path => [ $s->page_path ]
 
 =back
 
-In order to change the parameter for the new C<ht>, you can use this accessor before using the C<ht> (e.g. pass them to the new() method or set them in a C<OH_init>, or even using it inside the same Page Handler).
+You should use C<ht_new_args> at the very beginning of the process, for any argument but 'filename'.
 
     # set args in the new instance statement
     my $webapp = My::WebApp
-                ->new( ht_new_args => { path => '/my/path',    # override defaults
+                ->new( ht_new_args => { path => ['/my/path'],  # override defaults
                                         die_on_bad_params => 0,
                                         cache => 1
                                       }
@@ -242,21 +263,42 @@ In order to change the parameter for the new C<ht>, you can use this accessor be
     # or in the OH_init handler
     sub OH_init {
         my $s = shift;
-        $s->ht_new_args( path => '/my/path',     # override defaults
+        $s->ht_new_args( path => ['/my/path'],   # override defaults
                          die_on_bad_params => 0,
                          cache => 1 );
     }
-    
+
+B<Note about custom filenames>: it is preferable to avoid to explicitly set the filename argument and let the default do it for you. Anyway if you still need to set the filename, you must know that you have to reset it if you C<switch_to()> another page AFTER setting it.
+
     # and/or in a Page Handler
     sub PH_specialPage {
         my $s = shift;
         $s->ht_new_args( filename => 'special.tmp')     # override defaults
-        $s->ht->param( mySomething => 'something' );
+        $s->ht_param( mySomething => 'something' );
     }
 
 
 B<Note>: You can completely override the creation of the internal object by overriding the C<ht_new()> method.
 
+=head2 ht_param
+
+This property group accessor handles the HTML::Template parameters that are internally passed to the C<ht> object before the output production. Use it to collect the params that will be passed to the object.
+
+B<Note>: This group accessor has been added in the 1.2 version in order to avoid to use the C<ht> property:
+
+   # deprecated
+   $s->ht->param(...)
+   # OK
+   $s->ht_param(...)
+
+=head2 ht
+
+This property is used internally, so you usually don't need to use it directly in your code. In the rare case you need to use it (e.g. if you need to use C<HTML::Template::query()> method), you should use it after any other C<switch_to()> calls, or you should explicitly undef it just before the C<switch_to()> call.
+
+B<Note>: You can change the default arguments that are internally used to create the object by using the C<ht_new_args> group accessor, or you can completely override the creation of the internal object by overriding the C<ht_new()> method.
+
+B<Advanced Note>: Unlike other template object, the C<HTML::Template> object needs to know the filename at the moment of its creation. That restriction considerably limits the possibility to switch_to another page if and when the C<ht> property has been already used (i.e. the C<HTML::Template> object has been already created). For this reason, the C<ht> property is preferably used only as an ending point (i.e. after we know what is the ultimate page to serve).
+
 =head2 CBF changed property defaults
 
 =head3 CBF page_suffix
@@ -271,7 +313,19 @@ This module sets the default of the C<page_content> to the template output produ
 
 =head2 ht_new()
 
-This method is not intended to be used directly in your CBB. It is used internally to initialize and returns the C<HTML::Template> object, but you need to know how it does its job.  If you need some more customization you can redefine the method in your CBB.
+This method is not intended to be used directly in your CBB. It is used internally to initialize and returns the C<HTML::Template> object, but you need to know how it does its job. If you need some more customization you can redefine the method in your CBB.
+
+=head2 switch_to overriding
+
+If (and only if) your application has to switch_to after using the C<ht> property quite often, it may be handy to implement the following C<switch_to()> in your own CBB:
+
+  sub switch_to {
+     my $s = shift;
+   $s->ht_new_args( {filename =>$_[0].$s->page_suffix} )
+          if $s{ht_new_args}{filename};
+   $s->ht = undef;
+   $s->CGI::Builder::switch_to(@_);
+  }
 
 =head1 AVOIDING MISTAKES
 
@@ -297,3 +351,7 @@ See L<CGI::Builder/"SUPPORT">.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as perl itself.
 
+=head1 CREDITS
+
+Thanks to Rob Arnold for his testing and suggestions.
+
